/*

	WindowBlinds UIS2 Dll Plugin DEMO v1.1

     1999 Stardock Systems, Inc


	All this code is copyright and should not be used for ANY other purpose other
	than making a UIS2 plugin for use with WindowBlinds.


	This plugin is a clock which can be embedded into the titlebar
	It has been designed to only show when the item is active.


*/

#include <windows.h>
#include <wingdi.h>
#include <stdio.h>
#include <time.h>


/*
	WindowBlinds itself exports some functions for us to use

*/

typedef LPSTR (CALLBACK* GETUISNAME) (void);
typedef void (CALLBACK* SETALPHA) (int, int);


GETUISNAME GetUIS;
GETUISNAME GetWBDIR;

SETALPHA SetButtonAlpha;


/* 

	Global vars for this plugin

*/

UINT timer;
HFONT font;
HBITMAP bm;
BITMAP bm2;
int delay;
int alpha = 255;
int button;
int alphaInc;
BOOL up = FALSE;
HWND last;

int minAlpha;
int maxAlpha;

/*

	DLLMain


	Do your allocations & deallocations in here

*/

BOOL APIENTRY DllMain(PVOID hModule, ULONG ulReason, PCONTEXT pctx )
{
    BYTE str[255];
	BYTE fontName[255];
	HINSTANCE hDLL;

	if ( ulReason == DLL_PROCESS_ATTACH ) 
	{	
		/*

			This lets us get data from

		*/	

		hDLL = GetModuleHandle("WBLIND.DLL");
		GetUIS = (GETUISNAME)GetProcAddress(hDLL,"GetUIS");
		GetWBDIR = (GETUISNAME)GetProcAddress(hDLL,"GetWBDIR");
		SetButtonAlpha = (SETALPHA)GetProcAddress(hDLL,"SetButtonAlpha");


		if (GetUIS!=NULL)
		{
			// Obtain the UIS filename
			strcpy((LPSTR) str, GetUIS());
		}
		
		delay = GetPrivateProfileInt("Clock","UpdateTimer",900,(LPSTR) str);
		button = GetPrivateProfileInt("Clock","ButtonNo",0,(LPSTR) str);

		alphaInc = GetPrivateProfileInt("Clock","AlphaInc",1,(LPSTR) str);


		minAlpha = GetPrivateProfileInt("Clock","minAlpha",5,(LPSTR) str);

		maxAlpha = GetPrivateProfileInt("Clock","maxAlpha",255,(LPSTR) str);

		// Get the fontname
		GetPrivateProfileString("Clock","FontName","Arial",(LPSTR) fontName,255,(LPSTR) str);
	
		font = CreateFont(GetPrivateProfileInt("Clock","FontSize",15,(LPSTR) str), 0, 0,0 ,GetPrivateProfileInt("Clock","FontWeight",700,(LPSTR) str), 0, 0, 0, 0, OUT_TT_PRECIS, CLIP_TT_ALWAYS, PROOF_QUALITY   , DEFAULT_PITCH, (LPSTR) fontName);


		GetPrivateProfileString("Clock","Background","",(LPSTR) fontName,255,(LPSTR) str);

		// Check filename length

		if (strlen((LPSTR) fontName)>3)
		{

			strcpy((LPSTR) str, GetWBDIR());

			strcat((LPSTR) str,"\\");
			strcat((LPSTR) str,(LPSTR) fontName);

			bm = (HBITMAP)LoadImage(NULL, 
								(LPCTSTR) str, 
								IMAGE_BITMAP,
								0,
								0,
								LR_LOADFROMFILE| LR_LOADMAP3DCOLORS);

			GetObject(bm, sizeof(bm2), &bm2);
		}

		

	}	

	if ( ulReason == DLL_PROCESS_DETACH ) 
	{
		KillTimer(last, timer);
		DeleteObject(font);
		DeleteObject(bm);
	}
	return TRUE;
}


VOID CALLBACK TimerProc( HWND hwnd, UINT uMsg, UINT idEvent, DWORD dwTime )
{
	// This is our timer proc

	KillTimer(hwnd, timer);
	
	if (button > 0)
	{
		if (up)
			alpha+= alphaInc;
		else
			alpha-= alphaInc;
	
		if (alpha >= maxAlpha)
		{
			up = FALSE;
			alpha = maxAlpha;
		}
		if (alpha < minAlpha)
		{
			up = TRUE;
			alpha = minAlpha;
		}
		SetButtonAlpha(button, alpha);
	//	Beep(0,0);
	}

	
	// Force WB to redraw the window
	SendMessage(hwnd,WM_NULL,600,321);
}

/*

  Drawing code


  This is called by WindowBlinds when it needs to be redraw.

  Thanks to the flexibility of the UIS2 engine, whatever you draw
  will be processed like a normal bitmap.  So if you make any bits
  RGB(255,0,255) then they will be transparent.  You can also use
  the alpha blending option and your output will be blended!

  The size of the button can now be specified by the width & height
  lines in the UIS file.  See the UIS2 documentation for more details

*/

BOOL WBDraw (HWND hWnd, HDC DC, RECT* r, int state, BOOL active)
{

	/*

		hWnd = Handle to window
		DC = Handle to Device Context
		r = Dimensions of item
		state = > 0 then its pressed, else not pressed
		active = This is TRUE when the window is active and FALSE when inactive

	*/


	HBRUSH b, bOld;
	HBITMAP oldBM;
	HFONT fOld;
	HPEN p, pOld;
	HDC memDC;
	time_t ltime;
	char tmpbuf[128];



	if (active)
	{
		// Window is active so the timer is set
		last = hWnd;
		timer = SetTimer(hWnd, 1, delay, &TimerProc);
	}

	b = CreateSolidBrush(RGB(255,0,255));
	p = GetStockObject(NULL_PEN);

	// Draw it
	bOld = SelectObject(DC, b);
	pOld = SelectObject(DC, p);
	fOld = SelectObject(DC,font);

	Rectangle(DC,r->left, r->top, r->right+1, r->bottom+1);

	if (active)
	{
		if (bm!=NULL)
		{
			memDC = CreateCompatibleDC(DC);
			oldBM = SelectObject(memDC,bm);		
			StretchBlt(DC,r->left, r->top,r->right+1, r->bottom+1,memDC,0,0,bm2.bmWidth, bm2.bmHeight,SRCCOPY);
			DeleteDC(memDC);
			SelectObject(memDC,oldBM);
		}
	
		_tzset();
		_strtime( tmpbuf );    
		SetBkMode(DC,TRANSPARENT);
		SetTextColor(DC,GetSysColor(COLOR_CAPTIONTEXT));
		DrawText(DC, (LPSTR) tmpbuf, strlen((LPSTR) tmpbuf), r, DT_CENTER);
	}

	SelectObject(DC, bOld);
	SelectObject(DC, pOld);

	SelectObject(DC,fOld);

	DeleteObject(b);
	
	return TRUE;
}

BOOL WBPress (HWND hWnd, int x, int y, int state)
{

	// x & y are the position of the pointer
	// best to ignore them - For future use

	// The user has clicked on the button.  It has not yet been released

	return TRUE;
}

BOOL WBRelease (HWND hWnd, int x, int y, int state)
{

	// x & y are the position of the pointer
	// best to ignore them - For future use

	// The user has clicked on the button.  Button released while the pointer is over item
	// So item has been clicked

	// We shall beep just to indicate something

	Beep(1000,500);

	return TRUE;
}

BOOL WBQuery (HWND hWnd)
{

	// This function is for future use

	// For when we support timers etc

	return FALSE;
}